# @HEADER
# ************************************************************************
#
#            TriBITS: Tribal Build, Integrate, and Test System
#                    Copyright 2013 Sandia Corporation
#
# Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
# the U.S. Government retains certain rights in this software.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the Corporation nor the names of the
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# ************************************************************************
# @HEADER

##############################################################################
#
# CMake variable for use by Trilinos/Percept clients.
#
# Do not edit: This file was generated automatically by CMake.
#
##############################################################################

if(CMAKE_VERSION VERSION_LESS 3.3)
  set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
    "Percept requires CMake 3.3 or later for 'if (... IN_LIST ...)'"
    )
  set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
  return()
endif()
cmake_minimum_required(VERSION 3.3...3.23.0)

## ---------------------------------------------------------------------------
## Compilers used by Trilinos/Percept build
## ---------------------------------------------------------------------------

set(Percept_CXX_COMPILER "/usr/bin/mpicxx")

set(Percept_C_COMPILER "/usr/bin/mpicc")

set(Percept_Fortran_COMPILER "/usr/bin/mpif90")
# Deprecated!
set(Percept_FORTRAN_COMPILER "/usr/bin/mpif90") 


## ---------------------------------------------------------------------------
## Compiler flags used by Trilinos/Percept build
## ---------------------------------------------------------------------------

## Give the build type
set(Percept_CMAKE_BUILD_TYPE "RELEASE")

## Set compiler flags, including those determined by build type
set(Percept_CXX_FLAGS [[ -O3 -DNDEBUG]])

set(Percept_C_FLAGS [[  -march=x86-64 -mtune=generic -O2 -pipe -fno-plt -fexceptions         -Wp,-D_FORTIFY_SOURCE=2 -Wformat -Werror=format-security         -fstack-clash-protection -fcf-protection -flto=auto -O3 -DNDEBUG]])

set(Percept_Fortran_FLAGS [[ -O3]])
# Deprecated
set(Percept_FORTRAN_FLAGS [[ -O3]])

## Extra link flags (e.g., specification of fortran libraries)
set(Percept_EXTRA_LD_FLAGS [[]])

## This is the command-line entry used for setting rpaths. In a build
## with static libraries it will be empty.
set(Percept_SHARED_LIB_RPATH_COMMAND "-Wl,-rpath,/usr/lib")
set(Percept_BUILD_SHARED_LIBS "ON")

set(Percept_LINKER /usr/bin/ld)
set(Percept_AR /usr/bin/ar)

## ---------------------------------------------------------------------------
## Set library specifications and paths
## ---------------------------------------------------------------------------

## Base install location (if not in the build tree)
set(Percept_INSTALL_DIR "/usr")

## List of package libraries
set(Percept_LIBRARIES Percept::all_libs)

## ---------------------------------------------------------------------------
## MPI specific variables
##   These variables are provided to make it easier to get the mpi libraries
##   and includes on systems that do not use the mpi wrappers for compiling
## ---------------------------------------------------------------------------

set(Percept_MPI_LIBRARIES "")
set(Percept_MPI_LIBRARY_DIRS "")
set(Percept_MPI_INCLUDE_DIRS "")
set(Percept_MPI_EXEC "/usr/bin/mpiexec")
set(Percept_MPI_EXEC_MAX_NUMPROCS "4")
set(Percept_MPI_EXEC_NUMPROCS_FLAG "-np")

## ---------------------------------------------------------------------------
## Set useful general variables
## ---------------------------------------------------------------------------

# Enables/Disables for upstream package dependencies
set(Percept_ENABLE_SEACASIoss ON)
set(Percept_ENABLE_STKUtil ON)
set(Percept_ENABLE_STKIO ON)
set(Percept_ENABLE_STKMesh ON)
set(Percept_ENABLE_STKExprEval ON)
set(Percept_ENABLE_STKSearch ON)
set(Percept_ENABLE_STKTransfer ON)
set(Percept_ENABLE_Intrepid ON)
set(Percept_ENABLE_OpenNURBS OFF)

# Exported cache variables
set(Percept_ENABLE_DEBUG "OFF")
set(PERCEPT_DEBUG "OFF")
set(Percept_ENABLE_GEOMETRY "OFF")
set(STK_PERCEPT_HAS_GEOMETRY "OFF")
set(Percept_ENABLE_LITE "OFF")
set(STK_PERCEPT_LITE "OFF")
set(Percept_ENABLE_ACIS "OFF")
set(HAVE_ACIS "OFF")

# Include configuration of dependent packages
if (NOT TARGET SEACASIoss::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../SEACASIoss/SEACASIossConfig.cmake")
endif()
if (NOT TARGET STKUtil::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKUtil/STKUtilConfig.cmake")
endif()
if (NOT TARGET STKIO::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKIO/STKIOConfig.cmake")
endif()
if (NOT TARGET STKMesh::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKMesh/STKMeshConfig.cmake")
endif()
if (NOT TARGET STKExprEval::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKExprEval/STKExprEvalConfig.cmake")
endif()
if (NOT TARGET STKSearch::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKSearch/STKSearchConfig.cmake")
endif()
if (NOT TARGET STKTransfer::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../STKTransfer/STKTransferConfig.cmake")
endif()
if (NOT TARGET Intrepid::all_libs)
  include("${CMAKE_CURRENT_LIST_DIR}/../Intrepid/IntrepidConfig.cmake")
endif()

# Import Percept targets
include("${CMAKE_CURRENT_LIST_DIR}/PerceptTargets.cmake")

# Standard TriBITS-compliant external package variables
set(Percept_IS_TRIBITS_COMPLIANT TRUE)
set(Percept_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE "${CMAKE_CURRENT_LIST_FILE}")
set(Percept_TRIBITS_COMPLIANT_PACKAGE_CONFIG_FILE_DIR "${CMAKE_CURRENT_LIST_DIR}")


## ----------------------------------------------------------------------------
## Create deprecated non-namespaced library targets for backwards compatibility
## ----------------------------------------------------------------------------

set(Percept_EXPORTED_PACKAGE_LIBS_NAMES "percept")

foreach(libname IN LISTS Percept_EXPORTED_PACKAGE_LIBS_NAMES)
  if (NOT TARGET ${libname})
    add_library(${libname} INTERFACE IMPORTED)
    target_link_libraries(${libname}
       INTERFACE Percept::${libname})
    set(deprecationMessage
      "WARNING: The non-namespaced target '${libname}' is deprecated!"
      "  If always using newer versions of the project 'Trilinos', then use the"
      " new namespaced target 'Percept::${libname}', or better yet,"
      " 'Percept::all_libs' to be less sensitive to changes in the definition"
      " of targets in the package 'Percept'.  Or, to maintain compatibility with"
      " older or newer versions the project 'Trilinos', instead link against the"
      " libraries specified by the variable 'Percept_LIBRARIES'."
      )
    string(REPLACE ";" "" deprecationMessage "${deprecationMessage}")
    set_target_properties(${libname}
      PROPERTIES DEPRECATION "${deprecationMessage}" )
  endif()
endforeach()
